#include "MZCompRefl.h"

// Main model routine

void reflct(const RealArray& energyArray, const RealArray& params,
	     int spectrumNumber, RealArray& flux, RealArray& fluxErr, 
	     const string& initString)
{

   //     Driver for angle-dependent reflection from a neutral medium.
   //     See Magdziarz & Zdziarski, 1995, MNRAS.

   //     The output spectrum is the sum of the input (the original contents
   //     of Photar) and the reflection component.
   //     The reflection component alone can be obtained
   //     for scale (see below) = rel_refl < 0. Then the actual
   //     reflection normalization is |scale|. Note that you need to
   //     change then the limits of rel_refl. The range of rel_refl in that case
   //     should exclude zero (as then the direct component appears).

   //     Version with variable iron abundance and new opacities of Balucinska
   //     & McCammon (1992, and 1994, private communication). As expected in AGNs,
   //     H and He are assumed to be fully ionized.

   //     This version allows for changes of the vector 'ear' between subsequent
   //       calls.


   //     number of model parameters: 5
   //     1: scale, scaling factor for reflection; if <0, no direct component
   //        (scale=1 for isotropic source above disk)
   //     2: redshift, z
   //     3: abundance of elements heavier than He relative to
   //        the solar abundances
   //     4: iron abundance relative to the solar iron abundance
   //     5: cosine of inclination angle
   //     algorithm:
   //          a[E*(1+z)]=E^{-Gamma}*exp(-E/E_c)+scale*reflection
   //     Normalization is the photon flux at 1 keV (photons keV^-1 cm^-2 s^-1)
   //     of the cutoff power law only (without reflection)
   //     and in the earth frame.

   // also uses the following values which can be set using xset
   //   REFLECT_MAX_E      maximum energy for which to calculate the output spectrum
   //   REFLECT_DELTA_Y    delta y used for Greens' fn. integration
   //   REFLECT_INT_ORDER  order for Greens' fn. integration - overrides delta y
   //   REFLECT_PRECISION  Romberg's method precision for Greens' fn. integration
   //                      - overrides delta y and order.
   
   using namespace XSutility;

   static bool isFirst = true;
   if (isFirst)
   {
      string msg("Compton reflection from neutral medium.");
      msg += "\nSee help for details.\nIf you use results of this model in a paper,";
      msg += "\nplease refer to Magdziarz & Zdziarski 1995 MNRAS, 273, 837";
      xs_write(const_cast<char*>(msg.c_str()), 5);
      isFirst = false;
   }
   
   static int nesave = -1;
   const int Ne = static_cast<int>(energyArray.size()) - 1;
   static RealArray Sptot;
   static RealArray Spinc;
   static RealArray X;

   if (Ne != nesave)
   {
       Sptot.resize(Ne);
       Spinc.resize(Ne);
       X.resize(Ne);
       nesave = Ne;
   }

   // parameters

   Real Scale = params[0];
   Real zshift = 1.0 + params[1];
   Real Abund = log10(params[2]);
   Real FeAbund  = log10(params[3]);
   Real cosIncl = params[4];
   if ( cosIncl < 0.05 ) cosIncl = 0.05;
   if ( cosIncl > 0.95 ) cosIncl = 0.95;

   //  x is in the source frame and in units of m_e c^2.

   Real factor = zshift/2.0/511.0;
   for (int i=0; i<Ne; ++i)
     X[i] = (energyArray[i+1]+energyArray[i]) * factor;
      
   // Generate the spinc array from the input photar array.
   
   for (int i=0; i<Ne; ++i) {
     if (energyArray[i+1] != energyArray[i]) {
       Real avgE = (energyArray[i+1]+energyArray[i])/2.;
       Spinc[i] = flux[i]*avgE*avgE/(energyArray[i+1]-energyArray[i]);
     }
   }

   //     x is the energy array (units m_e c^2)
   //     spinc is the input spectrum array (E F_E)
   //     spref is the reflected spectrum array (E F_E)
   //     sptot is the total spectrum array (E F_E), = spinc if no reflection
   //     all dimensions = Ne

   // check whether the user has defined a maximum X that they want calculated

   Real Xmax(-1.0);
   string pname = "REFLECT_MAX_E";
   string pvalue = FunctionUtility::getModelString(pname);
   if ( pvalue.length() && pvalue != FunctionUtility::NOT_A_KEY() ) {
     std::istringstream iss(pvalue);
     if ( !(iss >> Xmax) || !iss.eof() ) {
       std::ostringstream err;
       err << "Invalid " << "REFLECT_MAX_E value: " << pvalue;
       xs_write(const_cast<char*>(err.str().c_str()), 10);
     }
   }
   Xmax /= 511.0;

   // calculate the total flux including direct and Compton reflection (non-relativistic
   // and relativistic).
   
   Real Temp(0.0);
   Real Xi(0.0);

   calcCompReflTotalFlux("REFLECT", Scale, cosIncl, Abund, FeAbund, Temp, 
			 Xi, Xmax, X, Spinc, Sptot);
   

   // convert back to xspec internal units
   
   for (int i=0; i<Ne; ++i) {
     Real avgE = (energyArray[i+1]+energyArray[i])/2.;
     flux[i] = Sptot[i]*(energyArray[i+1]-energyArray[i])/(avgE*avgE);
   }
                   
}

